var o=Object.defineProperty;var v=Object.getOwnPropertyDescriptor;var d=Object.getOwnPropertyNames;var p=Object.prototype.hasOwnProperty;var T=(t,e)=>{for(var n in e)o(t,n,{get:e[n],enumerable:!0})},E=(t,e,n,r)=>{if(e&&typeof e=="object"||typeof e=="function")for(let s of d(e))!p.call(t,s)&&s!==n&&o(t,s,{get:()=>e[s],enumerable:!(r=v(e,s))||r.enumerable});return t};var y=t=>E(o({},"__esModule",{value:!0}),t);var M={};T(M,{TypedEventTarget:()=>i});module.exports=y(M);var i=class extends EventTarget{dispatchTypedEvent(e,n){return super.dispatchEvent(n)}};0&&(module.exports={TypedEventTarget});
